import * as util from "util";

export class SunfishError extends Error {
    constructor(message: string = "") {
        super(`Something smells fishy!\n${message}`);
    }
}

export class UnknownInstructionError extends SunfishError {
    constructor(badInstruction: string) {
        super(`${badInstruction}" is not defined`);
    }
}

export class EmptyCodeBoxError extends SunfishError {
    constructor() {
        super("Cannot have an empty CodeBox (No room for the fish to survive)");
    }
}

export class FailedToParseCodeBoxError extends SunfishError {
    constructor(badCodeBoxString: string|string[]|string[][]) {
        super("Failed to parse CodeBox input.\n"+util.inspect(badCodeBoxString));
    }
}

export class EmptyStackError extends SunfishError {
    constructor() {
        super("Unable to perform this operation on an empty stack");
    }
}

export class WrongStackSizeError extends SunfishError {
    constructor(elementsWanted: number, stackSize: number) {
        super(`Failed to perform an operation on ${elementsWanted} on a stack with only ${stackSize} elements`);
    }
}

export class StackOutOfBoundsError extends SunfishError {
    constructor(badIndex: number) {
        super(`The stack pointer has gone out of bounds at index ${badIndex}`);
    }
}